import 'dart:io';

import 'package:braintree_flutter/model/drop_in_result.dart';
import 'package:braintree_flutter/model/google_payment.dart';
import 'package:braintree_flutter/model/single_payment_item.dart';
import 'package:flutter/services.dart';

import 'apple_payment.dart';

class BraintreeDropInRequest {
  static const MethodChannel _channel =
      const MethodChannel('braintree_flutter');

  /// Authorization allowing this client to communicate with Braintree.
  /// Either [clientToken] or [tokenizationKey] must be set.
  String clientToken;

  /// Authorization allowing this client to communicate with Braintree.
  /// Either [clientToken] or [tokenizationKey] must be set.
  String tokenizationKey;

  /// Amount for the transaction. This is only used for 3D secure verfications.
  String amount;

  /// If the Drop-in should collect and return device data for fraud prevention.
  bool collectDeviceData;

  /// If 3D Secure has been enabled in the control panel and an amount is specified in
  /// [amount], Drop-In will request a 3D Secure verification for any new cards added by the user.
  bool requestThreeDSecureVerification;

  GooglePaymentDropIn googlePaymentDropIn;
  ApplePaymentDropIn applePaymentDropIn;

  /// Whether Venmo should be enabled.
  bool venmoEnabled;

  /// Whether cards should be enabled.
  bool cardEnabled;

  /// Whether the card number should be masked if the field is not focused.
  bool maskCardNumber;

  /// Whether the security code should be masked during input.
  bool maskSecurityCode;

  /// Whether customers should be allowed to manage their vaulted payment methods.
  bool vaultManagerEnabled;

  BraintreeDropInRequest({
    this.clientToken,
    this.tokenizationKey,
    this.amount,
    this.collectDeviceData = false,
    this.requestThreeDSecureVerification = false,
    this.googlePaymentDropIn,
    this.applePaymentDropIn,
    this.venmoEnabled = true,
    this.cardEnabled = true,
    this.maskCardNumber = false,
    this.maskSecurityCode = false,
    this.vaultManagerEnabled = false,
  });

  Map<String, dynamic> toMap(BraintreeDropInRequest request) {
    var map = {
      'tokenizationKey': request.tokenizationKey,
      'clientToken': request.clientToken,
      'amount': request.amount,
      'collectDeviceData': request.collectDeviceData ?? false,
      'requestThreeDSecureVerification':
          request.requestThreeDSecureVerification ?? false,
      'googlePaymentEnabled': request.googlePaymentDropIn != null,
      'googlePaymentRequest_totalPrice': request.googlePaymentDropIn != null
          ? request.googlePaymentDropIn.totalPrice
          : null,
      'googlePaymentRequest_currencyCode': request.googlePaymentDropIn != null
          ? request.googlePaymentDropIn.currencyCode
          : null,
      'googlePaymentRequest_billingAddressRequired':
          request.googlePaymentDropIn != null
              ? (request.googlePaymentDropIn.billingAddressRequired ?? false)
              : null,
      'googlePaymentRequest_googleMerchantID':
          request.googlePaymentDropIn != null
              ? request.googlePaymentDropIn.googleMerchantID
              : null,
      'applePaymentEnabled': request.applePaymentDropIn != null,
      'applePaymentRequest_totalPrice': request.applePaymentDropIn != null
          ? request.applePaymentDropIn.totalPrice
          : null,
      'applePaymentRequest_currencyCode': request.applePaymentDropIn != null
          ? request.applePaymentDropIn.currencyCode
          : null,
      'applePaymentRequest_countryCode': request.applePaymentDropIn != null
          ? request.applePaymentDropIn.countryCode
          : null,
      'applePaymentRequest_billingAddressRequired':
          request.applePaymentDropIn != null
              ? (request.applePaymentDropIn.billingAddressRequired ?? false)
              : null,
      'venmoEnabled': request.venmoEnabled ?? false,
      'cardEnabled': request.cardEnabled ?? false,
      'maskCardNumber': request.maskCardNumber ?? false,
      'maskSecurityCode': request.maskSecurityCode ?? false,
      'vaultManagerEnabled': request.vaultManagerEnabled ?? false,
    };
    return map;
  }

  Map<String, dynamic> ItemMap(SinglePaymentItem item) {
    var map = {"label": item.label, "price": item.price};
    return map;
  }

  Future<String> start(BraintreeDropInRequest request) async {
    var result = null;
    if (Platform.isAndroid) {
      request.applePaymentDropIn = null;
      result = await _channel.invokeMethod<Map>(
        'startDropIn',
        toMap(request),
      );
      if (result == null) {
        return null;
      }
    } else if (Platform.isIOS) {
      request.googlePaymentDropIn = null;
      if (request.applePaymentDropIn == null ||
          request.applePaymentDropIn.items == null ||
          request.applePaymentDropIn.items.length == 0) {
        return null;
      }
      List<Map<String, dynamic>> list = new List();
      for (SinglePaymentItem item in request.applePaymentDropIn.items) {
        list.add(ItemMap(item));
      }

      result = await _channel.invokeMethod<Map>(
        'startDropIn',
        [toMap(request), list],
      );
      if (result == null) {
        return null;
      }
    }
    return (result["token"]);
  }

  BraintreeDropInResult _resultFromMap(Map map) {
    return BraintreeDropInResult(
      paymentMethodNonce: BraintreePaymentMethodNonce(
        nonce: map['paymentMethodNonce_nonce'],
        typeLabel: map['paymentMethodNonce_typeLabel'],
        description: map['paymentMethodNonce_description'],
        isDefault: map['paymentMethodNonce_isDefault'],
      ),
      deviceData: map['deviceData'],
    );
  }
}
